local ENABLED = nil;

function HealPointsFrame_OnLoad()
	local _, className = UnitClass("player");
	if (className == "PALADIN" or className == "PRIEST" or className == "DRUID" or className == "SHAMAN") then
		ENABLED = 1;
	end

	if (ENABLED) then
		SlashCmdList["HEALPOINTSCOMMAND"] = HealPoints_SlashHandler;
		SLASH_HEALPOINTSCOMMAND1 = "/healpoints";

		-- Register events
		this:RegisterEvent("UNIT_AURA");
		this:RegisterEvent("UNIT_LEVEL");
		this:RegisterEvent("UNIT_MAXMANA");
		this:RegisterEvent("UNIT_STATS");
		this:RegisterEvent("LEARNED_SPELL_IN_TAB");
		this:RegisterEvent("CHARACTER_POINTS_CHANGED");
	
		local oldFunc = HealPointsBS_Update;
		HealPointsBS_Update = function () 
			oldFunc(); 
			if (CharacterFrame:IsVisible()) then
				HealPointsFrame_SetPoints();
			end
		end

		DEFAULT_CHAT_FRAME:AddMessage("HealPoints loaded.",1,1,1);
	else
		DEFAULT_CHAT_FRAME:AddMessage("HealPoints not loaded - not a healing class",1,1,1);
	end
end

function HealPoints_SlashHandler(msg)
	if (msg == 'bscan') then
		local _, className = UnitClass("player");
		DEFAULT_CHAT_FRAME:AddMessage("Special set/equip bonuses detected by HealPoints:",1,1,1);		
		DEFAULT_CHAT_FRAME:AddMessage("==========================================",1,1,1);		
		DEFAULT_CHAT_FRAME:AddMessage("# % mana regeneration while casting: "..HealPointsBS:GetBonus('CASTINGREG'),1,1,1);		
		if (className == "PALADIN") then
			DEFAULT_CHAT_FRAME:AddMessage("# Reduced casting time Holy Light: 0."..HealPointsBS:GetBonus('CASTINGHOLYLIGHT'),1,1,1);		
		elseif (className == "PRIEST") then
			DEFAULT_CHAT_FRAME:AddMessage("# Reduced casting time Flash Heal: 0."..HealPointsBS:GetBonus('CASTINGFLASHHEAL'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Increased duration Renew: "..HealPointsBS:GetBonus('DURATIONRENEW'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Increased Holy spell crit chance: "..HealPointsBS:GetBonus('HOLYCRIT'),1,1,1);		
		elseif (className == "DRUID") then
			DEFAULT_CHAT_FRAME:AddMessage("# Reduced casting time Regrowth: 0."..HealPointsBS:GetBonus('CASTINGREGROWTH'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Increased duration Rejuvenation: "..HealPointsBS:GetBonus('DURATIONREJUV'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Increased Nature spell crit chance: "..HealPointsBS:GetBonus('NATURECRIT'),1,1,1);		
		elseif (className == "SHAMAN") then
			DEFAULT_CHAT_FRAME:AddMessage("# Reduced casting time Chain Heal: 0."..HealPointsBS:GetBonus('CASTINGCHAINHEAL'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Increased effect from Chain Heal jumps: "..HealPointsBS:GetBonus('IMPCHAINHEAL'),1,1,1);		
			DEFAULT_CHAT_FRAME:AddMessage("# Increased Nature spell crit chance: "..HealPointsBS:GetBonus('NATURECRIT'),1,1,1);		
		end		
	else
		if(HealPointsCalculator:IsVisible()) then
			HealPointsCalculator:Hide();
		else
			HealPointsCalculator:Show();
		end
	end
end

function HealPointsFrame_OnEvent(event)
	if (not CharacterFrame:IsVisible() or ENABLED == nil) then
		return;
	end
	-- Do nothing if event target is not player
	if ((	event == "UNIT_AURA" or 
			event == "UNIT_LEVEL" or 
			event == "UNIT_MAXMANA" or 
			event == "UNIT_STATS") 
			and not UnitIsUnit(arg1, "player")) then
		return;
	end	
	HealPointsFrame_SetPoints();
end

function HealPointsFrame_OnEnter()
	if (ENABLED) then
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		GameTooltip:SetText("HealPoints "..format("%5.0f", this.healPoints),
								HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("Mana regeneration while casting/5s:",
										format("%5.0f", this.manaRegenCasting),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("Normal mana regeneration/5s:",
										format("%5.0f", this.manaRegenNormal),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("+Healing from items:",
										format("%5.0f", this.healing),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("Chance to crit with healing spells:",
										format("%.2f%%", this.spellCrit),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Hint: Click to show the HealPoints calculator",
								GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b);
		GameTooltip:Show();
	end
end

function HealPointsFrame_OnShow()
	if (ENABLED) then
		HealPointsFrame_SetPoints();
	end
end

function HealPointsFrame_OnClick()
	if (HealPointsCalculator:IsVisible()) then
		HealPointsCalculator:Hide();
	elseif (ENABLED) then
		HealPointsCalculator:Show();
	end
end

function HealPointsFrame_SetPoints()
	local unit = "player";
	local label = getglobal("HealPointsFrameLabel");
	local text = getglobal("HealPointsFrameStatText");
	local frame = getglobal("HealPointsFrame");

	local manaRegenNormal, manaRegenCasting = HealPointsCalculator_ComputeManaRegen();
	local healing = HealPointsCalculator_ComputeHealing();
	local spellCrit = HealPointsCalculator_ComputeSpellCrit();
	local mana = UnitManaMax(unit);
	local healPoints = HealPointsCalculator_ComputeHealpoints(healing, spellCrit, mana, manaRegenCasting, manaRegenNormal);
	
	frame.healPoints = healPoints;
	frame.manaRegenCasting = manaRegenCasting;
	frame.manaRegenNormal = manaRegenNormal;
	frame.healing = healing;
	frame.spellCrit = spellCrit;

	label:SetText("HealPoints:");
	text:SetText(format("%5.0f", healPoints));
end
