--------------------------------------------------
-- BonusScanner v0.9.5 beta
-- by Crowley <crowley@headshot.de>
--
-- get the latest version here:
-- http://ui.worldofwar.net/ui.php?id=1461
-- http://www.curse-gaming.com/mod.php?addid=2384
--
-- Modified for HealPoints by Eridan
--------------------------------------------------

HEALPOINTSBS_PATTERN_SETNAME = "^(.*) %(%d/%d%)$";
HEALPOINTSBS_PATTERN_GENERIC_PREFIX = "^%+(%d+)%%?(.*)$";
HEALPOINTSBS_PATTERN_GENERIC_SUFFIX = "^(.*)%+(%d+)%%?$";

HealPointsBS = {
	bonuses = {};
	bonuses_details = {};

	active = nil;
	temp = { 
		sets = {},
		set = "",
		slot = "",
		bonuses = {},
		details = {}
	};

	types = {
		"INT", 			-- intellect
		"SPI", 			-- spirit

		"SPELLCRIT",	-- chance to crit with spells
		"HEAL",			-- healing 
		"HOLYCRIT", 	-- chance to crit with holy spells

		"MANAREG",		-- mana regeneration per 5 sec.
		"MANA",			-- mana points

		-- Added
		"CASTINGREG", 		-- % mana regeneration while casting 			
		"NATURECRIT", 		-- chance to crit with nature spells      
		"CASTINGREGROWTH", 	-- reduced casting time Regrowth     
		"CASTINGHOLYLIGHT", -- reduced casting time Holy Light  
		"CASTINGFLASHHEAL", -- reduced casting time Flash Heal  
		"CASTINGCHAINHEAL", -- reduced casting time Chain Heal
		"DURATIONREJUV",  	-- increased duration Rejuvenation  
		"DURATIONRENEW",    -- increased duration Renew
		"MANAREGNORMAL",	-- normal mana regeneration per 5 sec.
		"IMPCHAINHEAL"      -- increased effect from chain heal jumps
	};

	slots = {
		"Head",
		"Neck",
		"Shoulder",
		"Shirt",
		"Chest",
		"Waist",
		"Legs",
		"Feet",
		"Wrist",
		"Hands",
		"Finger0",
		"Finger1",
		"Trinket0",
		"Trinket1",
		"Back",
		"MainHand",
		"SecondaryHand",
		"Ranged",
		"Tabard",
	};
}

-- Update function to hook into. 
-- Gets called, when Equipment changes (after UNIT_INVENTORY_CHANGED)
function HealPointsBS_Update()
end

function HealPointsBS:GetBonus(bonus)
	if(HealPointsBS.bonuses[bonus]) then
		if (type(HealPointsBS.bonuses[bonus]) == "string") then
			return tonumber(HealPointsBS.bonuses[bonus]);
		else
			return HealPointsBS.bonuses[bonus];
		end
	end;
	return 0;
end

function HealPointsBS:GetSlotBonuses(slotname)
	local i, bonus, details;
	local bonuses = {};
	for bonus, details in HealPointsBS.bonuses_details do
		if(details[slotname]) then
			bonuses[bonus] = details[slotname];
		end
	end
	return bonuses;
end

function HealPointsBS:GetBonusDetails(bonus)
	if(HealPointsBS.bonuses_details[bonus]) then
		return HealPointsBS.bonuses_details[bonus];
	end;
	return {};
end

function HealPointsBS:GetSlotBonus(bonus, slotname)
	if(HealPointsBS.bonuses_details[bonus]) then
		if(HealPointsBS.bonuses_details[bonus][slotname]) then
			return HealPointsBS.bonuses_details[bonus][slotname];
		end;
	end;
	return 0;
end


function HealPointsBS:OnLoad()
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");
end

function HealPointsBS:OnEvent()
	if (event == "PLAYER_ENTERING_WORLD") then
		HealPointsBS.active = 1;
		this:RegisterEvent("UNIT_INVENTORY_CHANGED");
	end
	if (((event == "PLAYER_ENTERING_WORLD") or (event == "UNIT_INVENTORY_CHANGED")) and HealPointsBS.active) then
		HealPointsBS:ScanEquipment();
		HealPointsBS_Update();
	end
	if (event == "PLAYER_LEAVING_WORLD") then
		this:UnregisterEvent("UNIT_INVENTORY_CHANGED");
	end	
end


function HealPointsBS:ScanEquipment()
	local slotid, slotname, hasItem, i;

	HealPointsBS.temp.bonuses = {};
	HealPointsBS.temp.details = {};
	HealPointsBS.temp.sets = {};
	HealPointsBS.temp.set = "";

	for i, slotname in HealPointsBS.slots do
		slotid, _ = GetInventorySlotInfo(slotname.. "Slot");
		hasItem = HealPointsBSTooltip:SetInventoryItem("player", slotid);
	
		if ( hasItem ) then
			HealPointsBS.temp.slot = slotname;
			HealPointsBS:ScanTooltip();
			-- if set item, mark set as already scanned
			if(HealPointsBS.temp.set ~= "") then
				HealPointsBS.temp.sets[HealPointsBS.temp.set] = 1;
			end;
		end
	end
	HealPointsBS.bonuses = HealPointsBS.temp.bonuses;
	HealPointsBS.bonuses_details = HealPointsBS.temp.details;
end

function HealPointsBS:ScanItem(itemlink)
	local name = GetItemInfo(itemlink);
	if(name) then
		HealPointsBS.temp.bonuses = {};
		HealPointsBS.temp.sets = {};
		HealPointsBS.temp.set = "";
		HealPointsBS.temp.slot = "";
		HealPointsBSTooltip:SetHyperlink(itemlink);
		HealPointsBS:ScanTooltip();
		return HealPointsBS.temp.bonuses;
	end
	return false;
end

function HealPointsBS:ScanTooltip()
	local tmpTxt, line;
	local lines = HealPointsBSTooltip:NumLines();

	for i=2, lines, 1 do
		tmpText = getglobal("HealPointsBSTooltipTextLeft"..i);
		val = nil;
		if (tmpText:GetText()) then
			line = tmpText:GetText();
			HealPointsBS:ScanLine(line);
		end
	end
end

function HealPointsBS:AddValue(effect, value)
	local i,e;
	
	if(type(effect) == "string") then
		if(HealPointsBS.temp.bonuses[effect]) then
			HealPointsBS.temp.bonuses[effect] = HealPointsBS.temp.bonuses[effect] + value;
		else
			HealPointsBS.temp.bonuses[effect] = value;
		end
		
		if(HealPointsBS.temp.slot) then
			if(HealPointsBS.temp.details[effect]) then
				if(HealPointsBS.temp.details[effect][HealPointsBS.temp.slot]) then
					HealPointsBS.temp.details[effect][HealPointsBS.temp.slot] = HealPointsBS.temp.details[effect][HealPointsBS.temp.slot] + value;
				else
					HealPointsBS.temp.details[effect][HealPointsBS.temp.slot] = value;
				end
			else
				HealPointsBS.temp.details[effect] = {};
				HealPointsBS.temp.details[effect][HealPointsBS.temp.slot] = value;
			end
		end;
	else 
	-- list of effects
		if(type(value) == "table") then
			for i,e in effect do
				HealPointsBS:AddValue(e, value[i]);
			end
		else
			for i,e in effect do
				HealPointsBS:AddValue(e, value);
			end
		end
	end
end;

function HealPointsBS:ScanLine(line)
	local tmpStr, found;
	-- Check for "Equip: "
	if(string.sub(line,0,string.len(HEALPOINTSBS_PREFIX_EQUIP)) == HEALPOINTSBS_PREFIX_EQUIP) then

		tmpStr = string.sub(line,string.len(HEALPOINTSBS_PREFIX_EQUIP)+1);
		HealPointsBS:CheckPassive(tmpStr);

	-- Check for "Set: "
	elseif(string.sub(line,0,string.len(HEALPOINTSBS_PREFIX_SET)) == HEALPOINTSBS_PREFIX_SET
			and HealPointsBS.temp.set ~= "" 
			and not HealPointsBS.temp.sets[HealPointsBS.temp.set]) then

		tmpStr = string.sub(line,string.len(HEALPOINTSBS_PREFIX_SET)+1);
		HealPointsBS.temp.slot = "Set";
		HealPointsBS:CheckPassive(tmpStr);

	-- any other line (standard stats, enchantment, set name, etc.)
	else
		-- Check for set name
		_, _, tmpStr = string.find(line, HEALPOINTSBS_PATTERN_SETNAME);
		if(tmpStr) then
			HealPointsBS.temp.set = tmpStr;
		else
			found = HealPointsBS:CheckGeneric(line);
			if(not found) then
				HealPointsBS:CheckOther(line);
			end;
		end
	end
end;


-- Scans passive bonuses like "Set: " and "Equip: "
function HealPointsBS:CheckPassive(line)
	local i, p, value, found;

	found = nil;
	for i,p in HEALPOINTSBS_PATTERNS_PASSIVE do
		_, _, value = string.find(line, "^" .. p.pattern);
		if(value) then
			HealPointsBS:AddValue(p.effect, value)
			found = 1;
		end
	end
	if(not found) then
		HealPointsBS:CheckGeneric(line);
	end
end


-- Scans generic bonuses like "+3 Intellect" or "Arcane Resistance +4"
function HealPointsBS:CheckGeneric(line)
	local value, token, pos, tmpStr, found;

	-- split line at "/" for enchants with multiple effects
	found = false;
	while(string.len(line) > 0) do
		pos = string.find(line, "/", 1, true);
		if(pos) then
			tmpStr = string.sub(line,1,pos-1);
			line = string.sub(line,pos+1);
		else
			tmpStr = line;
			line = "";
		end

		-- trim line
	    tmpStr = string.gsub( tmpStr, "^%s+", "" );
   	    tmpStr = string.gsub( tmpStr, "%s+$", "" );
       	tmpStr = string.gsub( tmpStr, "%.$", "" );

		_, _, value, token = string.find(tmpStr, HEALPOINTSBS_PATTERN_GENERIC_PREFIX);
		if(not value) then
			_, _,  token, value = string.find(tmpStr, HEALPOINTSBS_PATTERN_GENERIC_SUFFIX);
		end
		if(token and value) then
			-- trim token
		    token = string.gsub( token, "^%s+", "" );
    	    token = string.gsub( token, "%s+$", "" );
	       	token = string.gsub( token, "%.$", "" );
	
			if(HealPointsBS:CheckToken(token,value)) then
				found = true;
			end
		end
	end
	return found;
end


-- Identifies simple tokens like "Intellect" and composite tokens like "Fire damage" and 
-- add the value to the respective bonus. 
-- returns true if some bonus is found
function HealPointsBS:CheckToken(token, value)
	local i, p, s1, s2;
	
	if(HEALPOINTSBS_PATTERNS_GENERIC_LOOKUP[token]) then
		HealPointsBS:AddValue(HEALPOINTSBS_PATTERNS_GENERIC_LOOKUP[token], value);
		return true;
	else
		s1 = nil;
		s2 = nil;
		for i,p in HEALPOINTSBS_PATTERNS_GENERIC_STAGE1 do
			if(string.find(token,p.pattern,1,1)) then
				s1 = p.effect;
			end
		end	
		for i,p in HEALPOINTSBS_PATTERNS_GENERIC_STAGE2 do
			if(string.find(token,p.pattern,1,1)) then
				s2 = p.effect;
			end
		end	
		if(s1 and s2) then
			HealPointsBS:AddValue(s1..s2, value);
			return true;
		end 
	end
	return false;
end

-- Last fallback for non generic enchants, like "Mana Regen x per 5 sec."
function HealPointsBS:CheckOther(line)
	local i, p, value, start, found;

	for i,p in HEALPOINTSBS_PATTERNS_OTHER do
		start, _, value = string.find(line, "^" .. p.pattern);
		if(start) then
			if(p.value) then
				HealPointsBS:AddValue(p.effect, p.value)
			elseif(value) then
				HealPointsBS:AddValue(p.effect, value)
			end
			return true;
		end
	end
	return false;
end
